pub use tecplot::get_MM;
pub use tecplot::get_index_from_value;
pub use tecplot::get_one_element_WH;
pub use tecplot::get_tecplot_SN;
pub use tecplot::write_image_CM;

mod tecplot {
    use serde::{Deserialize, Serialize};
    use serde_json::json;
    use serde_json::{Result, Value};
    use std::env::Args;

    use regex::Regex;
    use std::fs::OpenOptions;
    use std::io::{Read, Write};
    use std::{
        fs::File,
        // io::Read,
        io::{BufRead, BufReader},
    };
    extern crate image;
    use image::ImageFormat;

    use cloudmap::cm3;
    use cloudmap::cm3_arrow;
    use cloudmap::cm3_blue;
    use cloudmap::cm3_mud;

    pub fn get_one_element_WH(
        list: &Vec<Vec<f64>>,
        cols: usize,
        rows: usize,
        elx: usize,
        ely: usize,
    ) -> (f64, f64) {
        let x = 0;
        let y = 1;
        let el_value_3 = &list[((y) * (cols) + x) as usize];

        // let el_value_2 = &list[((y) * (cols) + x + 1) as usize];

        // let el_value_0 = &list[((y - 1) * (cols) + x) as usize];

        let el_value_1 = &list[((y - 1) * (cols) + x + 1) as usize];

        let el_3_x = *el_value_3.get(elx).unwrap();
        let el_1_x = *el_value_1.get(elx).unwrap();
        let el_3_y = *el_value_3.get(ely).unwrap();
        let el_1_y = *el_value_1.get(ely).unwrap();

        (el_1_x - el_3_x, el_3_y - el_1_y)
    }
    fn get_one_element_SN(
        list: &Vec<Vec<f64>>,
        max: f64,
        min: f64,
        x: usize,
        y: usize,
        cols: usize,
        rows: usize,
        index: usize,
        elx: usize,
        ely: usize,
    ) -> (f64, f64, f64, f64, f64, f64) {
        let el_value_3 = &list[((y) * (cols) + x) as usize];

        let el_value_2 = &list[((y) * (cols) + x + 1) as usize];

        let el_value_0 = &list[((y - 1) * (cols) + x) as usize];

        let el_value_1 = &list[((y - 1) * (cols) + x + 1) as usize];

        let el_3_x = *el_value_3.get(elx).unwrap();
        let el_1_x = *el_value_1.get(elx).unwrap();
        let el_3_y = *el_value_3.get(ely).unwrap();
        let el_1_y = *el_value_1.get(ely).unwrap();

        let value_3 = (el_value_3.get(index).unwrap() - min) / (max - min);

        let value_0 = (el_value_0.get(index).unwrap() - min) / (max - min);

        let value_1 = (el_value_1.get(index).unwrap() - min) / (max - min);

        let value_2 = (el_value_2.get(index).unwrap() - min) / (max - min);

        (
            value_0,
            value_1,
            value_2,
            value_3,
            el_1_x - el_3_x,
            el_3_y - el_1_y,
        )
    }
    pub fn write_image_CM(
        list: &Vec<Vec<f64>>,
        max: f64,
        min: f64,
        cols: usize,
        rows: usize,
        pn: usize,
        cut: f64,
        filename: &str,
        color: &str,
        index: usize,
        elx: usize,
        ely: usize,
        w: usize,
        h: usize,
    ) {
        let pnw = (pn * w) as f64;
        let pnh = (pn * h) as f64;
        let imgx = ((cols - 1) * w * pn) as u32;
        let imgy = ((rows - 1) * h * pn) as u32;
        let AB = (h as f64 / w as f64);
        // Create a new ImgBuf with width: imgx and height: imgy
        let mut imgbuf = image::ImageBuffer::new(imgx, imgy);

        for x in 0..cols - 1 {
            //  print!("行：   {}\n", x);
            for y in 0..rows - 1 {
                let (ca, cb, cc, cd, el_x, el_y) = get_one_element_SN(
                    list,
                    max,
                    min,
                    x,
                    rows - 1 - y,
                    cols,
                    rows,
                    index,
                    elx,
                    ely,
                );

                for rx in 0..pnw as usize {
                    for ry in 00..pnh as usize {
                        let pixel = imgbuf.get_pixel_mut(
                            (x * pnw as usize + rx) as u32,
                            (y * pnh as usize + ry) as u32,
                        );
                        let image::Rgba(data) = *pixel;
                        let mut cmv: [u32; 4];
                        // if ry * h / w > (pn * w - rx) {
                        // let ab = if rx == 0 {
                        //     if ry >= 0 {
                        //         1.0
                        //     } else {
                        //         0.0
                        //     }
                        // } else {
                        //     ry as f64 / rx as f64
                        // };
                        // if ab < AB {
                        if ry as f64 * (1.0/AB) >= pnw - rx as f64 {
                            // print!("ab:{ab},AB:{AB},ca:{ca},cb:{cb},cc:{cc}\n");
                            let u = rx as f64 / pnw as f64;
                            let v = 1.0 - (ry as f64 / pnh as f64);
                            if color == "mud" {
                                cmv = cm3_mud(u, v, ca, cb, cc, cut, 12);
                            } else if color == "blue" {
                                cmv = cm3_blue(u, v, ca, cb, cc, cut, 12);
                            } else {
                                cmv = cm3(u, v, ca, cb, cc, cut, 12);
                            }
                            *pixel = image::Rgba([
                                cmv[0] as u8,
                                cmv[1] as u8,
                                cmv[2] as u8,
                                cmv[3] as u8,
                            ]);
                        } else {
                            let u = (pnh - ry as f64) / pnh as f64;
                            let v = (rx) as f64 / pnw as f64;
                            if color == "mud" {
                                cmv = cm3_mud(u, v, ca, cd, cc, cut, 12);
                            } else if color == "blue" {
                                cmv = cm3_blue(u, v, ca, cd, cc, cut, 12);
                            } else {
                                cmv = cm3(u, v, ca, cd, cc, cut, 12);
                            }

                            *pixel = image::Rgba([
                                cmv[0] as u8,
                                cmv[1] as u8,
                                cmv[2] as u8,
                                cmv[3] as u8,
                            ]);
                        }
                    }
                }
            }
        }
        // print!("ready to  write cm\n");
        // Save the image as “fractal.png”, the format is deduced from the path
        imgbuf.save(filename).unwrap();
    }

    pub fn get_MM(list: &Vec<Vec<f64>>, index: usize) -> (f64, f64) {
        let mut max = 0.;
        let mut min = 0.;

        let mut flag_first = false;

        let mut new_list: Vec<Vec<f64>> = Vec::new();

        for one in list {
            let one_value = *one.get(index).unwrap();
            if flag_first == false {
                max = one_value;
                min = one_value;
                flag_first = true;
            }
            if min > one_value {
                min = one_value;
            }
            if max < one_value {
                max = one_value;
            }
        }

        (max, min)
    }

    pub fn get_index_from_value(data: Vec<(usize, String)>, value: &str) -> usize {
        for (k, v) in data {
            print!("{k},{v}\n");
            if v.as_str() == value {
                return k;
            }
        }
        999999
    }
    pub fn get_tecplot_SN(filename: &str) -> (Vec<Vec<f64>>, Vec<(usize, String)>, u64, u64) {
        let file = File::open(filename).unwrap();
        let lines = BufReader::new(file).lines(); //遍历所有行
        let mut i: u64 = 0;

        let mut values_line = Vec::new();
        let mut flag_start_data = false;
        let mut list: Vec<Vec<f64>> = Vec::new();

        let mut total_I: u64 = 1;
        let mut total_J: u64 = 1;
        let mut ij: u64 = 0;

        let seperator = Regex::new(r"([ ]+)").expect("Invalid regex"); //多个空格的regex

        //遍历
        for line in lines {
            if ij == total_I * total_J {
                break;
            }
            if let Ok(data) = line {
                let line_space: Vec<&str> = seperator.split(data.as_str()).into_iter().collect();
                // print!("{flag_start_data} {:?}\n", line_space);
                if flag_start_data == true {
                    // print!("start_data =true {flag_start_data}\n");
                    let mut per_line = Vec::new();
                    for per_one in line_space {
                        per_line.push(per_one.parse::<f64>().unwrap());
                    }
                    list.push(per_line);
                    ij += 1;
                    let a = 1;
                }
                if flag_start_data == false {
                    // print!("start_data =false, {flag_start_data}\n");
                    if data.contains("I =") {
                        let line_space: Vec<&str> = data.as_str().split(",").collect();
                        for per_ij in line_space {
                            if per_ij.contains("I = ") {
                                let str_i: Vec<&str> = per_ij.split("I = ").collect();
                                let a = 1;
                                total_I = str_i[1].parse::<u64>().unwrap();
                                let a = 1;
                            } else if per_ij.contains("J = ") {
                                let str_i: Vec<&str> = per_ij.split("J = ").collect();
                                let a = 1;
                                total_J = str_i[1].parse::<u64>().unwrap();
                                let a = 1;
                            }
                            flag_start_data = true;
                        }
                    } else if data.contains("variables") {
                        let str_v = data.as_str().replace("\"", "");

                        let mut n_values_line: Vec<&str> =
                            seperator.split(str_v.as_str()).into_iter().collect();
                        n_values_line.remove(0);
                        n_values_line.remove(0);
                        let mut i_k: usize = 0;
                        for per_v in n_values_line {
                            let a = String::from(per_v);
                            values_line.push((i_k, a));
                            i_k += 1;
                        }
                    }
                }
            }
        }
        (list, values_line, total_I, total_J)
    }
}
